
title 'SASHELP.CLASS ... NO MISSING DATA';
proc print data=sashelp.class;
run;

* CHANGE SOME VALUES TO MISSING;
data test;
set sashelp.class;
if ranuni(0) lt .3 then call missing (name);
if ranuni(0) lt .5 then call missing (weight);
run;

title 'DATA SET TEST ... WITH MISSING DATA';
proc print data=test;
run;

title;

* FORMATS TO GROUP "REAL" AND MISSING VALUES;
proc format;
value $c ' ' = '0' other = '1';
value  n  .  = '0' other = '1';
run;

* CREATE TABLE FOR ONE VARIABLE;
* USE PROC FREQ OPTION OUT=;
proc freq data=test;
tables weight / missing out=tables;
format weight  n.;
run;

proc print data=tables;
*format _all_;
run;

proc contents data=tables;
ods select variables;
run;

* CREATE TABLES FOR ALL VARIABLES;
* USE PROC FREQ OPTION OUT=;
proc freq data=test;
tables _all_ / missing out=tables;
format _numeric_ n. _character_ $c.;
run;

proc print data=tables;
*format _all_;
run;

proc contents data=tables;
ods select variables;
run;

* CREATE TABLE FOR ONE VARIABLE;
* USE ODS OUTPUT;
ods output onewayfreqs=tables;
ods listing close;

proc freq data=test;
tables weight /  missing;
format weight n.;
run;

ods output close;
ods listing;

proc print data=tables;
*format _all_;
run;

proc contents data=tables;
ods select variables;
run;

* CREATE TABLES FOR ALL VARIABLES;
* USE ODS OUTPUT;
*ods output onewayfreqs=tables;
ods output onewayfreqs=tables (keep=table f_: frequency percent);
ods listing close;

proc freq data=test;
tables _all_ /  missing;
format _numeric_ n. _character_ $c.;
run;

ods output close;
ods listing;

proc print data=tables;
*format _all_;
run;

proc contents data=tables;
ods select variables;
run;




* USE SOME SAS FUNCTIONS TO GRAB VALUES FROM THE DATA SET;
data report;
length var $32;

do until (last.table);
   set tables;
   by table notsorted;
   level = coalescec(of f_:);
   select (level);
     when ('0') do; miss = frequency; p_miss = percent; end;
     when ('1') do; ok   = frequency; p_ok   = percent; end;
     otherwise;
   end;
end;

miss   = coalesce(miss,0);
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);
var    = scan(table,-1);

keep var miss ok p_:;
format miss ok comma7. p_: 5.1;
run;

title "MISSING DATA REPORT";
proc print data=report noobs;
var var miss p_miss ok p_ok;
run;
title;

ods listing close;
ods results off;
ods html file='z:\missing.html';

title "MISSING DATA REPORT";
proc print data=report noobs;
var var miss p_miss ok p_ok;
run;
title;

ods html close;
ods results on;
ods listing;


* another SASHELP data set ... it has MISSING data and labels;

title 'DATA SET SASHELP.HEART';
proc contents data=sashelp.heart;
ods select variables;
run;

proc print data=sashelp.heart (obs=50);
run;
title;

ods output onewayfreqs=tables (keep=table f_: frequency percent);
ods listing close;

proc freq data=sashelp.heart;
tables _all_ /  missing;
format _numeric_ n. _character_ $c.;
run;

ods output close;
ods listing;

data report;
length var $32;

do until (last.table);
   set tables;
   by table notsorted;
   level = coalescec(of f_:);
   select (level);
     when ('0') do; miss = frequency; p_miss = percent; end;
     when ('1') do; ok   = frequency; p_ok   = percent; end;
     otherwise;
   end;
end;

miss   = coalesce(miss,0);
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);
var    = scan(table,-1);

keep var miss ok p_:;
format miss ok comma7. p_: 5.1;
run;

title "MISSING DATA REPORT ... SASHELP.HEART";
ods listing close;
ods results off;
ods html file='z:\heart.html';
title;

proc print data=report noobs;
var var miss p_miss ok p_ok;
run;

ods html close;
ods results on;
ods listing;

* ADD LABELS;
* PUT VARIABEL NAMES AND LABELS INTO A DATA SET;
proc contents data=sashelp.heart
              noprint out=labels (keep=name label rename=(name=var) index=(var));
run;

title 'DATA SET LABELS (INDEXED)';
proc print data=labels;
run;

proc contents data=labels;
run;
title;

ods output onewayfreqs=tables (keep=table f_: frequency percent);
ods listing close;

proc freq data=sashelp.heart;
tables _all_ /  missing;
format _numeric_ n. _character_ $c.;
run;

ods output close;
ods listing;

data report;
length var $32;

do until (last.table);
   set tables;
   by table notsorted;
   level = coalescec(of f_:);
   select (level);
     when ('0') do; miss = frequency; p_miss = percent; end;
     when ('1') do; ok   = frequency; p_ok   = percent; end;
     otherwise;
   end;
end;

miss   = coalesce(miss,0);
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);
var    = scan(table,-1);
set labels key=var / unique;

keep var label miss ok p_:;
format miss ok comma7. p_: 5.1;
run;

ods listing close;
ods results off;
ods html file='z:\heart_labels.html';

title "MISSING DATA REPORT ... SASHELP.HEART";
proc print data=report uniform noobs;
var var label miss p_miss ok p_ok;
run;
title;

ods html close;
ods results on;
ods listing;
